var toString = Object.prototype.toString;
function isDate(val) {
    return toString.call(val) === '[object Date]';
}
function isPlainObject(val) {
    return toString.call(val) === '[object Object]';
}
function extend(to, from) {
    for (var key in from) {
        to[key] = from[key];
    }
    return to;
}
function deepMerge() {
    var objs = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        objs[_i] = arguments[_i];
    }
    var result = Object.create(null);
    objs.forEach(function (obj) {
        if (obj) {
            Object.keys(obj).forEach(function (key) {
                var val = obj[key];
                if (isPlainObject(val)) {
                    if (isPlainObject(result[key])) {
                        result[key] = deepMerge(result[key], val);
                    }
                    else {
                        result[key] = deepMerge(val);
                    }
                }
                else {
                    result[key] = val;
                }
            });
        }
    });
    return result;
}
function isFormData(val) {
    return typeof val !== 'undefined' && val instanceof FormData;
}
function isURLSearchParams(val) {
    return typeof val !== 'undefined' && val instanceof URLSearchParams;
}

function normalizeHeaderName(headers, normalizedName) {
    if (!headers) {
        return;
    }
    Object.keys(headers).forEach(function (name) {
        if (name !== normalizedName && name.toUpperCase() === normalizedName.toUpperCase()) {
            headers[normalizedName] = headers[name];
            delete headers[name];
        }
    });
}
function processHeaders(headers, data) {
    normalizeHeaderName(headers, 'Content-Type');
    if (isPlainObject(data)) {
        if (headers && !headers['Content-Type']) {
            headers['Content-Type'] = 'application/json;charset=utf-8';
        }
    }
    return headers;
}
function parseHeaders(headers) {
    var parsed = Object.create(null);
    if (!headers) {
        return parsed;
    }
    headers.split('\r\n').forEach(function (line) {
        var _a = line.split(':'), key = _a[0], vals = _a.slice(1);
        key = key.trim().toLocaleLowerCase();
        if (!key) {
            return;
        }
        var val = vals.join(':').trim();
        parsed[key] = val;
    });
    return parsed;
}
function flattenHeaders(headers, method) {
    if (!headers) {
        return headers;
    }
    headers = deepMerge(headers.common, headers[method], headers);
    var methodsToDelete = ['delete', 'get', 'head', 'options', 'post', 'put', 'patch', 'common'];
    methodsToDelete.forEach(function (method) {
        delete headers[method];
    });
    return headers;
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var AxiosError = /** @class */ (function (_super) {
    __extends(AxiosError, _super);
    function AxiosError(message, config, code, request, response) {
        var _this = _super.call(this, message) || this;
        _this.config = config;
        _this.code = code;
        _this.request = request;
        _this.response = response;
        _this.isAxiosError = true;
        // 根据ts的文档，当继承一些内置对象如Error、Map类的时候，继承后的类的实例，可能无法访问自身的方法
        // 同时，instanceof此类的实例，也不等于此类。解决办法就是在调用父类构造函数后，立即手动改变原型指向
        Object.setPrototypeOf(_this, AxiosError.prototype);
        return _this;
    }
    return AxiosError;
}(Error));
function createError(message, config, code, request, response) {
    var error = new AxiosError(message, config, code, request, response);
    return error;
}

/**
 * 判断是否同源请求
 * @param requestURL 请求的URL
 */
function isURLSameOrigin(requestURL) {
    var parsedOrigin = resolveURL(requestURL);
    return (parsedOrigin.protocol === currentOrigin.protocol && parsedOrigin.host === currentOrigin.host);
}
// 通过创建一个a标签，设置a标签的href属性为某个url，直接从a标签的protocol和host属性中读书协议、域名端口
// protocol表示协议如，'http:'
// host表示域名端口如，'127.0.0.1:8080'
var urlParsingNode = document.createElement('a');
var currentOrigin = resolveURL(window.location.href);
function resolveURL(url) {
    urlParsingNode.setAttribute('href', url);
    var protocol = urlParsingNode.protocol, host = urlParsingNode.host;
    return {
        protocol: protocol,
        host: host
    };
}
function encode(val) {
    // 将URL内容进行编码，但是将以下符号转换回来不编码
    // 带字母的部分正则加i标记不区分大小写
    return encodeURIComponent(val)
        .replace(/%40/g, '@')
        .replace(/%3A/ig, ':')
        .replace(/%24/g, '$')
        .replace(/%2C/ig, ',')
        .replace(/%20/g, '+')
        .replace(/%5B/ig, '[')
        .replace(/%5D/ig, ']');
}
/**
 * 构建get请求的url函数
 * 构建的规则：
 * 1、如果参数属性，对应的值是数组，如params:{foo: ['bar', 'baz']}，那么在url中的形式为：xxxx?foo[]=bar&foo[]=baz
 * 2、如果参数属性，对应的值是对象，则为stringify后的字符串，并且经过编码
 * 3、url中带有哈希值，即'#'符号，忽略url中'#'符号后面的内容
 * 4、url中若已有参数，则params中的参数跟在后面，否则url末尾加上'?'，并且跟上参数
 * 5、特殊的符号如'@'、':'、'$'、'['、']'、','转码回来，不显示编码后的代码，空格符用'+'号替代
 * 6、如果参数的属性，对应的值是日期对象，则值用toISOString()转换成字符串
 * 7、如果参数的属性，是null或者undefined，则url中不带上这个参数
 * @param url axios请求的url地址
 * @param params 请求的参数
 */
function buildURL(url, params, paramsSerializer) {
    if (!params) {
        return url;
    }
    var serializedParams;
    if (paramsSerializer) {
        serializedParams = paramsSerializer(params);
    }
    else if (isURLSearchParams(params)) {
        serializedParams = params.toString();
    }
    else {
        var parts_1 = [];
        Object.keys(params).forEach(function (key) {
            var val = params[key];
            // 如果属性的值是null，或者undefined，url中不会带上这个参数，直接跳过
            if (val === null || typeof val === 'undefined') {
                return;
            }
            var values = [];
            // 如果属性的值是一个数组，那么url中key的后面要跟上'[]'
            if (Array.isArray(val)) {
                values = val;
                key += '[]';
            }
            else {
                // 如果属性值不是数组，那么将它变成数组，方便处理
                values = [val];
            }
            values.forEach(function (val) {
                if (isDate(val)) {
                    val = val.toISOString();
                }
                else if (isPlainObject(val)) {
                    val = JSON.stringify(val);
                }
                parts_1.push(encode(key) + "=" + encode(val));
            });
        });
        serializedParams = parts_1.join('&');
    }
    // 如果上面的处理结果是空数组，那么join后是空字符串，不处理
    // 同时如果url里有哈希值，即myUrl#hash形式，哈希后面的是页面内锚点或跳转，不需要在请求url中跟上
    if (serializedParams) {
        var markIndex = url.indexOf('#');
        if (markIndex !== -1) {
            url = url.slice(0, markIndex);
        }
        // 如果这个url已经带有参数，那么它本身带有'?'，后面加上'&'并拼接
        // 否则，先加上'?'，并拼接
        url += (url.indexOf('?') === -1 ? '?' : '&') + serializedParams;
    }
    return url;
}
/**
 * 判断一个url是否是绝对地址
 * @param url URL地址
 */
function isAbsoluteURL(url) {
    // 正则含义：
    // 绝对地址的url，以类似'http:'或'https:'开头，后面跟两个斜线'//'；也可以直接两个斜线开头
    // 正则匹配字母开头，后面可以是0个或多个字母、数字、加号'+'、减号'-'、点号'.'，再跟冒号':'。这部分可能有也可能没有，所以跟问号表示0或1个
    // 斜线需要转义
    // 需要忽略大小写，正则表达式跟i标识符
    return /^([a-z][a-z\d\+\-\.]*:)?\/\//i.test(url);
}
/**
 * 将基础URL和相对路径URL拼成完整的URL
 * @param baseURL 基础URL
 * @param relativeURL 相对路径URL
 */
function combineURL(baseURL, relativeURL) {
    // 拼接时，删除baseURL末尾的斜线'/'，以及relativeURL开头的斜线'/'，两个再以斜线'/'拼接
    return relativeURL ? baseURL.replace(/\/+$/, '') + '/' + relativeURL.replace(/^\/+/, '') : baseURL;
}

var cookie = {
    read: function (name) {
        // 正则含义：
        // 相当于reg = /(^|;\s*)(name)=([^;]*)/
        // 即cookie项是要么在开头或者是分号后面开始，包含名称=xxxx的形式，=号后面跟着0个或多个非分号
        var match = document.cookie.match(new RegExp('(^|;\\s*)(' + name + ')=([^;]*)'));
        // 一个括号代表一个正则匹配项。match[0]代表整个匹配结果。那么name对应的内容，是在match[3]匹配项内
        return match ? decodeURIComponent(match[3]) : null;
    },
};

function xhr(config) {
    return new Promise(function (resolve, reject) {
        var _a = config.data, data = _a === void 0 ? null : _a, url = config.url, method = config.method, _b = config.headers, headers = _b === void 0 ? {} : _b, responseType = config.responseType, timeout = config.timeout, cancelToken = config.cancelToken, withCredentials = config.withCredentials, xsrfHeaderName = config.xsrfHeaderName, xsrfCookieName = config.xsrfCookieName, onDownloadProgress = config.onDownloadProgress, onUploadProgress = config.onUploadProgress, auth = config.auth, validateStatus = config.validateStatus;
        var request = new XMLHttpRequest();
        request.open(method.toUpperCase(), url, true);
        configureRequest();
        addEvents();
        processHeaders$$1();
        processCancel();
        request.send(data);
        /**
         * 对request对象进行一些配置
         */
        function configureRequest() {
            // 设置返回类型
            if (responseType) {
                request.responseType = responseType;
            }
            // 设置timeout超时的时间，如果没有设置timeout，就不需要赋值，默认为0，即一直等待不超时。单位: 毫秒
            if (timeout) {
                request.timeout = timeout;
            }
            // 跨域请求时，通过设置这个属性，来使请求带上所要请求的域下的cookie
            if (withCredentials) {
                request.withCredentials = withCredentials;
            }
        }
        /**
         * 为request对象添加各类事件监听
         */
        function addEvents() {
            /** 状态改变监听 */
            request.onreadystatechange = function handleLoad() {
                // 如果readyState不为4，表示请求未完成
                if (request.readyState !== 4) {
                    return;
                }
                // 当发生网络错误、超时错误时，status也是0
                if (request.status === 0) {
                    return;
                }
                // 请求完成，则执行以下代码
                var responseHeaders = parseHeaders(request.getAllResponseHeaders());
                var respnoseData = responseType && responseType !== 'text' ? request.response : request.responseText;
                var response = {
                    data: respnoseData,
                    status: request.status,
                    statusText: request.statusText,
                    headers: responseHeaders,
                    config: config,
                    request: request,
                };
                // 判断status状态码来决定是resolve或者reject
                handleResponse(response);
            };
            /** 网络错误监听 */
            request.onerror = function handleError() {
                reject(createError('Network Error', config, null, request));
            };
            /** 超时错误的处理 */
            request.ontimeout = function handleTimeout() {
                reject(createError("Timeout of " + timeout + " ms exceeded", config, 'ECONNABORTED', request));
            };
            // 下载进度监控事件
            if (onDownloadProgress) {
                request.onprogress = onDownloadProgress;
            }
            // 上传进度监控事件
            if (onUploadProgress) {
                request.upload.onprogress = onUploadProgress;
            }
        }
        /**
         * 对请求头headers进行处理
         */
        function processHeaders$$1() {
            // 如果数据是FormData的话，删除content-type的header，让浏览器自动添加
            if (isFormData(data)) {
                delete headers['Content-Type'];
            }
            // 如果携带cookie，或者是同源的，并且设置有xsrfCookieName；那么通过cookie读取，在header上设置它的值，作为后端需要的token传过去
            if ((withCredentials || isURLSameOrigin(url)) && xsrfCookieName) {
                var xsrfValue = cookie.read(xsrfCookieName);
                if (xsrfValue && xsrfHeaderName) {
                    headers[xsrfHeaderName] = xsrfValue;
                }
            }
            // 基本验证 的请求头设置
            if (auth) {
                headers['Authorization'] = 'Basic ' + btoa(auth.username + ':' + auth.password);
            }
            // 设置请求头headers
            Object.keys(headers).forEach(function (name) {
                if (data === null && name.toLocaleLowerCase() === 'content-type') {
                    delete headers[name];
                }
                else {
                    request.setRequestHeader(name, headers[name]);
                }
            });
        }
        /**
         * 处理取消的逻辑
         */
        function processCancel() {
            if (cancelToken) {
                // tslint:disable-next-line: no-floating-promises
                cancelToken.promise.then(function (reason) {
                    request.abort();
                    reject(reason);
                });
            }
        }
        /** 根据status状态码，决定是resolve或者reject */
        function handleResponse(response) {
            // validateStatus有默认值，如果使用者传入配置将其改为了null那么说明使用者希望所有的请求都视为合法的
            // 或者validateStatus返回为true，也视为合法。resolve返回结果
            if (!validateStatus || validateStatus(response.status)) {
                resolve(response);
            }
            else {
                reject(createError("Request failed with status code " + response.status, config, null, request, response));
            }
        }
    });
}

function transform(data, headers, fns) {
    if (!fns) {
        return data;
    }
    if (!Array.isArray(fns)) {
        fns = [fns];
    }
    fns.forEach(function (fn) {
        data = fn(data, headers);
    });
    return data;
}

function dispatchRequest(config) {
    throwIfCancellationRequested(config);
    processConfig(config);
    return xhr(config).then(function (res) {
        return transformResponseData(res);
    }, function (e) {
        if (e && e.response) {
            e.response = transformResponseData(e.response);
        }
        return Promise.reject(e);
    });
}
function processConfig(config) {
    config.url = transformURL(config);
    config.data = transform(config.data, config.headers, config.transformRequest);
    config.headers = flattenHeaders(config.headers, config.method);
}
function transformURL(config) {
    var url = config.url, params = config.params, paramsSerializer = config.paramsSerializer, baseURL = config.baseURL;
    if (baseURL && !isAbsoluteURL(url)) {
        url = combineURL(baseURL, url);
    }
    return buildURL(url, params, paramsSerializer);
}
function transformResponseData(res) {
    res.data = transform(res.data, res.headers, res.config.transformResponse);
    return res;
}
function throwIfCancellationRequested(config) {
    if (config.cancelToken) {
        config.cancelToken.throwIfRequested();
    }
}

var InterceptorManager = /** @class */ (function () {
    function InterceptorManager() {
        this.interceptors = [];
    }
    InterceptorManager.prototype.use = function (resolve, reject) {
        this.interceptors.push({
            resolve: resolve,
            reject: reject,
        });
        return this.interceptors.length - 1;
    };
    InterceptorManager.prototype.eject = function (id) {
        if (this.interceptors[id]) {
            this.interceptors[id] = null;
        }
    };
    InterceptorManager.prototype.forEach = function (fn) {
        this.interceptors.forEach(function (interceptor) {
            if (interceptor) {
                fn(interceptor);
            }
        });
    };
    return InterceptorManager;
}());

/**
 * 默认策略，如果用户有传入配置就用用户的，否则用默认配置
 * @param val1 默认配置
 * @param val2 用户传入的配置
 */
function defaultStrat(val1, val2) {
    return typeof val2 !== 'undefined' ? val2 : val1;
}
/**
 * 只从用户传递的配置中读取设置
 * @param val1 默认配置
 * @param val2 用户传入的配置
 */
function fromVal2Strat(val1, val2) {
    if (typeof val2 !== 'undefined') {
        return val2;
    }
}
/**
 * 深度合并复杂对象
 * 1、如果val2是普通对象，那么与val1合并
 * 2、如果val2不是undefined，并且不是对象，用val2的值
 * 3、如果val2是undefined，val1是普通对象，合并
 * 4、如果val2是undefined，并且val1不为undefined，用val1的值
 * @param val1 默认配置
 * @param val2 用户传入的配置
 */
function deepMergeStrat(val1, val2) {
    if (isPlainObject(val2)) {
        return deepMerge(val1, val2);
    }
    else if (typeof val2 !== 'undefined') {
        return val2;
    }
    else if (isPlainObject(val1)) {
        return deepMerge(val1);
    }
    else {
        return val1;
    }
}
// 合并策略的函数的映射集合
var strats = Object.create(null);
// 只从用户配置中读取的属性
var stratKeysFromVal2 = ['url', 'params', 'data'];
stratKeysFromVal2.forEach(function (key) {
    strats[key] = fromVal2Strat;
});
// 需要深度拷贝的属性
var stratKeysDeepMerge = ['headers', 'auth'];
stratKeysDeepMerge.forEach(function (key) {
    strats[key] = deepMergeStrat;
});
function mergeConfig(config1, config2) {
    if (!config2) {
        config2 = {};
    }
    var config = Object.create(null);
    for (var key in config2) {
        mergeField(key);
    }
    for (var key in config1) {
        if (!config2[key]) {
            mergeField(key);
        }
    }
    function mergeField(key) {
        var strat = strats[key] || defaultStrat;
        config[key] = strat(config1[key], config2[key]);
    }
    return config;
}

var Axios = /** @class */ (function () {
    function Axios(initConfig) {
        this.defaults = initConfig;
        this.interceptors = {
            request: new InterceptorManager(),
            response: new InterceptorManager(),
        };
    }
    // request支持两种传参方式，一种传config对象，一种传url和config对象
    // 因此，两个参数的类型需要为any，在代码中进一步判断
    Axios.prototype.request = function (url, config) {
        if (typeof url === 'string') {
            if (!config) {
                config = {};
            }
            config.url = url;
        }
        else {
            config = url;
        }
        // 合并传入的配置和默认配置
        config = mergeConfig(this.defaults, config);
        // 将method转换成小写
        config.method = config.method.toLowerCase();
        // 定义拦截器调用链的初始值，初始的时候没有拦截器，里面只有默认要做的事情，就是dispatchRequest
        var chain = [
            {
                resolve: dispatchRequest,
                reject: undefined,
            }
        ];
        /** 循环request和response拦截器管理实例，分别将拦截器添加到初始值的两边 */
        this.interceptors.request.forEach(function (interceptor) {
            chain.unshift(interceptor);
        });
        this.interceptors.response.forEach(function (interceptor) {
            chain.push(interceptor);
        });
        var promise = Promise.resolve(config);
        while (chain.length) {
            var _a = chain.shift(), resolve = _a.resolve, reject = _a.reject;
            promise = promise.then(resolve, reject);
        }
        return promise;
    };
    Axios.prototype.get = function (url, config) {
        return this._requestMethodWithoutData('get', url, config);
    };
    Axios.prototype.delete = function (url, config) {
        return this._requestMethodWithoutData('delete', url, config);
    };
    Axios.prototype.head = function (url, config) {
        return this._requestMethodWithoutData('head', url, config);
    };
    Axios.prototype.options = function (url, config) {
        return this._requestMethodWithoutData('options', url, config);
    };
    Axios.prototype.post = function (url, data, config) {
        return this._requestMethodWithData('post', url, data, config);
    };
    Axios.prototype.put = function (url, data, config) {
        return this._requestMethodWithData('put', url, data, config);
    };
    Axios.prototype.patch = function (url, data, config) {
        return this._requestMethodWithData('patch', url, data, config);
    };
    Axios.prototype.getUri = function (config) {
        config = mergeConfig(this.defaults, config);
        return transformURL(config);
    };
    Axios.prototype._requestMethodWithoutData = function (method, url, config) {
        return this.request(Object.assign(config || {}, {
            method: method,
            url: url,
        }));
    };
    Axios.prototype._requestMethodWithData = function (method, url, data, config) {
        return this.request(Object.assign(config || {}, {
            method: method,
            url: url,
            data: data,
        }));
    };
    return Axios;
}());

function transformRequest(data) {
    if (isPlainObject(data)) {
        return JSON.stringify(data);
    }
    return data;
}
function transformResponse(data) {
    if (typeof data === 'string') {
        try {
            data = JSON.parse(data);
        }
        catch (error) {
            // do nothing
        }
    }
    return data;
}

var defaults = {
    method: 'get',
    timeout: 0,
    headers: {
        common: {
            Accept: 'application/json, text/plain, */*'
        }
    },
    xsrfCookieName: 'XSRF-TOKEN',
    xsrfHeaderName: 'X-XSRF-TOKEN',
    transformRequest: [
        function (data, headers) {
            processHeaders(headers, data);
            return transformRequest(data);
        }
    ],
    transformResponse: [
        function (data) {
            return transformResponse(data);
        }
    ],
    validateStatus: function (status) {
        return status >= 200 && status < 300;
    }
};
var methodsNoData = ['delete', 'get', 'head', 'options'];
methodsNoData.forEach(function (method) {
    defaults.headers[method] = {};
});
var methodsWithData = ['post', 'put', 'patch'];
methodsWithData.forEach(function (method) {
    defaults.headers[method] = {
        'Content-Type': 'application/x-www-form-urlencoded'
    };
});

var Cancel = /** @class */ (function () {
    function Cancel(message) {
        this.message = message;
    }
    return Cancel;
}());
function isCancel(value) {
    return value instanceof Cancel;
}

// 注意，此处的Cancel不能从接口定义中去取，而要从类定义中拿。因为我们要使用Cancel
var CancelToken = /** @class */ (function () {
    function CancelToken(executor) {
        var _this = this;
        var resolvePromise;
        this.promise = new Promise(function (resolve) {
            resolvePromise = resolve;
        });
        executor(function (message) {
            if (_this.reason) {
                return;
            }
            _this.reason = new Cancel(message);
            resolvePromise(_this.reason);
        });
    }
    CancelToken.prototype.throwIfRequested = function () {
        if (this.reason) {
            throw this.reason;
        }
    };
    CancelToken.source = function () {
        var cancel;
        var token = new CancelToken(function (c) {
            cancel = c;
        });
        return { cancel: cancel, token: token };
    };
    return CancelToken;
}());

function createInstance(config) {
    var context = new Axios(config);
    var instance = Axios.prototype.request.bind(context);
    extend(instance, context);
    return instance;
}
// 此时的axios，是一个方法，同时也有其他get、post、delete等方法挂载其上
// 既可以用axios()的方式调用，也可以axios.get()的方式调用
var axios = createInstance(defaults);
axios.create = function create(config) {
    return createInstance(mergeConfig(defaults, config));
};
axios.CancelToken = CancelToken;
axios.Cancel = Cancel;
axios.isCancel = isCancel;
axios.all = function all(promises) {
    return Promise.all(promises);
};
axios.spread = function spread(callback) {
    return function wrap(arr) {
        return callback.apply(null, arr);
    };
};
axios.Axios = Axios;

export default axios;
//# sourceMappingURL=axios.es5.js.map
