/**
 * 判断是否同源请求
 * @param requestURL 请求的URL
 */
export declare function isURLSameOrigin(requestURL: string): boolean;
/**
 * 构建get请求的url函数
 * 构建的规则：
 * 1、如果参数属性，对应的值是数组，如params:{foo: ['bar', 'baz']}，那么在url中的形式为：xxxx?foo[]=bar&foo[]=baz
 * 2、如果参数属性，对应的值是对象，则为stringify后的字符串，并且经过编码
 * 3、url中带有哈希值，即'#'符号，忽略url中'#'符号后面的内容
 * 4、url中若已有参数，则params中的参数跟在后面，否则url末尾加上'?'，并且跟上参数
 * 5、特殊的符号如'@'、':'、'$'、'['、']'、','转码回来，不显示编码后的代码，空格符用'+'号替代
 * 6、如果参数的属性，对应的值是日期对象，则值用toISOString()转换成字符串
 * 7、如果参数的属性，是null或者undefined，则url中不带上这个参数
 * @param url axios请求的url地址
 * @param params 请求的参数
 */
export declare function buildURL(url: string, params?: any, paramsSerializer?: (params: any) => string): string;
/**
 * 判断一个url是否是绝对地址
 * @param url URL地址
 */
export declare function isAbsoluteURL(url: string): boolean;
/**
 * 将基础URL和相对路径URL拼成完整的URL
 * @param baseURL 基础URL
 * @param relativeURL 相对路径URL
 */
export declare function combineURL(baseURL: string, relativeURL?: string): string;
